@@ -60,6 +60,8 @@ def mkdb_oma(args):
     LOG.info('Building index')
     ki = Index(db, k=args.k, reduced_alphabet=args.reduced_alphabet, nthreads=1, hidden_taxa=hidden_taxa)
     ki.build_kmer_table()
+    db.add_metadata()
+    db.add_hogcounts()
 
     db.close()
     LOG.info('Done')
@@ -108,7 +110,7 @@ def search(args):
         seqs.append(str(rec.seq))
         if len(ids) == args.chunksize:
             ms.merge_search(
-                seqs=seqs, ids=ids, fasta_file=None, score=score, cum_mode='max', top_m_fams=100, top_n_fams=1, perm_nr=1, w_size=6, dist='poisson', fam_filter=np.array([], dtype=np.int64)
+                seqs=seqs, ids=ids, fasta_file=None, score=score, top_m_fams=100, top_n_fams=1, perm_nr=1, w_size=6, dist='poisson', fam_filter=np.array([], dtype=np.int64)
             )
             pbar.update(len(ids))
             df = ms.output_results(overlap=0, fst=0, sst=args.threshold, ref_taxon=args.reference_taxon)
@@ -121,7 +123,7 @@ def search(args):
     # final search
     if len(ids) > 0:
         ms.merge_search(
-            seqs=seqs, ids=ids, fasta_file=None, score=score, cum_mode='max', top_m_fams=100, top_n_fams=1, perm_nr=1, w_size=6, dist='poisson', fam_filter=np.array([], dtype=np.int64)
+            seqs=seqs, ids=ids, fasta_file=None, score=score, top_m_fams=100, top_n_fams=1, perm_nr=1, w_size=6, dist='poisson', fam_filter=np.array([], dtype=np.int64)
         )
         df = ms.output_results(overlap=0, fst=0, sst=args.threshold, ref_taxon=args.reference_taxon)
         pbar.update(len(ids))
@@ -130,3 +132,16 @@ def search(args):
 
     pbar.close()
     db.close()
+
+
+def info_db(args):
+    from .database import Database
+    with Database(args.db) as db:
+        print("=" * 80)
+        for k, v in db.get_metadata().items():
+            if isinstance(v, list):
+                if len(v) == 0:
+                    v = ['-']
+                v = ",".join(v)
+            print(f"  {k:23s}:{v!s:>40}")
+        print("=" * 80)
